import { world, system, EquipmentSlot, MolangVariableMap } from "@minecraft/server";
import { nameSpace } from "./namespace.js";
import * as guidebook from "./guidebook.js";
import * as loot from "./loot.js";
import { mobheadMenu } from "./menu/menu.js";

let interval = 2;
system.runInterval(ticker, interval);
let lastEntitiesInView = [];

function ticker() {
    let players = world.getPlayers();

    if (players.length > 0) {
        let entitiesInView = [];
        for (let player of players) {
            let playerEquipment = player.getComponent("minecraft:equippable");
            let mainhandItem = playerEquipment.getEquipment(EquipmentSlot.Mainhand);
            if (mainhandItem && mainhandItem.typeId == `${nameSpace}:head_wrench`) {
                let hits = player.getEntitiesFromViewDirection({
                    maxDistance: 3.5,
                    families: ["2tails_mobheads_head"],
                });
                if (hits.length > 0) {
                    for (let hit of hits) {
                        if (!entitiesInView.includes(hit.entity.id)) {
                            entitiesInView.push(hit.entity.id);
                            hit.entity.setProperty(`${nameSpace}:looking_at`, true);
                        }
                    }
                }
            }
            if (lastEntitiesInView.length > 0) {
                for (let entityId of lastEntitiesInView) {
                    if (!entitiesInView.includes(entityId)) {
                        let entity = world.getEntity(entityId);
                        if (entity) {
                            entity.setProperty(`${nameSpace}:looking_at`, false);
                        }
                    }
                }
            }
        }
        lastEntitiesInView = entitiesInView;
    }
}

// Give mob head on death by player
world.afterEvents.entityDie.subscribe((data) => {
    let damageSource = data.damageSource;
    let damagingEntity = damageSource.damagingEntity;
    let entity = data.deadEntity;
    if (entity != undefined && damagingEntity != undefined) {
        if (damagingEntity.typeId == "minecraft:player") {
            loot.dropHead(entity, damagingEntity);
        }
    }
});

// Punch (Attack) to Pick Up or Menu when holding Wrench
world.afterEvents.entityHitEntity.subscribe((data) => {
    let damagingEntity = data.damagingEntity;
    let entity = data.hitEntity;
    if (entity != undefined && damagingEntity != undefined) {
        let familyComponent = entity.getComponent("minecraft:type_family");
        if (familyComponent.hasTypeFamily(nameSpace + "_head")) {
            if (damagingEntity.typeId == "minecraft:player") {
                let playerEquipment = damagingEntity.getComponent("minecraft:equippable");
                let mainhandItem = playerEquipment.getEquipment(EquipmentSlot.Mainhand);
                if (mainhandItem && mainhandItem.typeId == nameSpace + ":head_wrench") {
                    mobheadMenu(damagingEntity, entity);
                } else {
                    entity.triggerEvent(`${nameSpace}:pickup`);
                }
            }
        }
    }
});

// Open menu from entity interact
world.afterEvents.dataDrivenEntityTrigger.subscribe((data) => {
    if (data.eventId == `${nameSpace}:menu`) {
        const players = data.entity.dimension.getPlayers({
            location: { x: data.entity.location.x, y: data.entity.location.y, z: data.entity.location.z },
            closest: 1,
            maxDistance: 5.5,
        });
        mobheadMenu(players[0], data.entity);
    }
});

// Rotate head to cardinal direction when placed
world.afterEvents.entitySpawn.subscribe((data) => {
    let entity = data.entity;
    if (entity != undefined) {
        let familyComponent = entity.getComponent("minecraft:type_family");
        if (!familyComponent) {
            return;
        }
        if (familyComponent.hasTypeFamily(nameSpace + "_head")) {
            let rotation = entity.getRotation();
            if (rotation) {
                let y_rotation = Math.round(rotation.y / 5) * 5;
                if (y_rotation < 0) {
                    y_rotation += 360;
                }
                entity.setRotation({ x: 0, y: 0 });
                entity.setProperty(`${nameSpace}:y_rotation`, y_rotation);
            }
        }
    }
});
