import { world, system, EquipmentSlot, EntityComponentTypes } from "@minecraft/server";
import { nameSpace } from "./namespace.js";
import { mobs } from "./mobs.js";

export function dropHead(entity, player) {
    // BUG: Can't get dimension or location from wither
    if (entity.isValid) {
        let error = false;
        let entityType = entity.typeId;
        let dimension = undefined;
        let location = { x: 0, y: 0, z: 0 };
        try {
            dimension = entity.dimension;
            location = entity.location;
        } catch (err) {
            error = true;
        }
        if (error == true) {
            error = false;
            try {
                dimension = player.dimension;
                location = player.location;
            } catch (err) {
                error = true;
            }
        }
        if (error == false && dimension != undefined) {
            try {
                let type = entityType.replace("minecraft:", "");

                if (entity.hasComponent("minecraft:is_baby") && type != "tadpole") {
                    return;
                } else {
                    if (willDrop(type, player)) {
                        let lootTable = getLootTable(type, entity);
                        dimension.runCommand(
                            `loot spawn ${location.x} ${location.y} ${location.z} loot "2tails/mobheads/${lootTable}"`
                        );
                    }
                }
            } catch (err) {
                error = true;
            }
        }
    } else {
        return;
    }
}

function willDrop(type, player) {
    let rand = Math.round(Math.random() * 100) / 100;
    let chance = getChance(type);
    let looting = hasLooting(player);
    let total = Math.round((chance + (chance * (15 * looting)) / 100) * 100) / 100;
    if (rand <= total) {
        return true;
    } else {
        return false;
    }
}

function hasLooting(player) {
    let playerEquipment = player.getComponent("minecraft:equippable");
    let mainhandItem = playerEquipment.getEquipment(EquipmentSlot.Mainhand);
    if (mainhandItem) {
        let enchantments = mainhandItem.getComponent("minecraft:enchantable");
        if (enchantments) {
            let looting = enchantments.getEnchantment("minecraft:looting");
            if (looting) {
                return looting.level;
            }
        }
    }

    return 0;
}

function getChance(type) {
    let chance = 1;
    if (mobs[type] && mobs[type]["chance"]) {
        chance = mobs[type]["chance"];
    }
    return chance;
}

function getLootTable(type, entity) {
    // BUG: Wolves in flat world are just wolf (no variant)
    let typeVariant = type;
    if (type == 'villager_v2') {
        typeVariant = 'v';
    } else if (type == 'zombie_villager_v2') {
        typeVariant = 'zv';
    }
    switch (type) {
        case "strider":
            if (entity.hasComponent(EntityComponentTypes.IsShaking)) {
                return typeVariant + "_shaking";
            }
            break;
        default:
            let variant = entity.getComponent(EntityComponentTypes.Variant);
            let markVariant = entity.getComponent(EntityComponentTypes.MarkVariant);
            if (variant && markVariant && mobs[type]["variant"] && mobs[type]["mark_variant"]) {
                let value = mobs[type]["mark_variant"][markVariant.value];
                switch (value) {
                    case 'desert' :
                        value = 'des';
                        break;
                    case 'jungle' :
                        value = 'jun';
                        break;
                    case 'plains' :
                        value = 'pla';
                        break;
                    case 'savanna' :
                        value = 'sav';
                        break;
                    case 'snow' :
                        value = 'sno';
                        break;
                    case 'swamp' :
                        value = 'swa';
                        break;
                    case 'taiga' :
                        value = 'tai';
                        break;
                }

                typeVariant += "_" + value;
                value = mobs[type]["variant"][variant.value];
                typeVariant += "_" + value;
            } else {
                if (variant && mobs[type]["variant"]) {
                    let value = mobs[type]["variant"][variant.value];
                    typeVariant += "_" + value;
                }
                if (markVariant && mobs[type]["mark_variant"]) {
                    let value = mobs[type]["mark_variant"][markVariant.value];
                    typeVariant += "_" + value;
                }
                let skinId = entity.getComponent(EntityComponentTypes.SkinId);
                if (skinId && mobs[type]["skin_id"]) {
                    let value = mobs[type]["skin_id"][skinId.value];
                    typeVariant += "_" + value;
                }
                let color = entity.getComponent(EntityComponentTypes.Color);
                if (color && mobs[type]["color"]) {
                    let value = mobs[type]["color"][color.value];
                    typeVariant += "_" + value;
                }
            }
            return typeVariant;
            break;
    }
    return typeVariant;
}
